@@ -19,21 +19,20 @@
 
 #include "bookmarks.h"
 
-#include <sys/types.h>
-
-#include <ctype.h> /* isalnum() */
 #include <stddef.h> /* NULL */
+#include <stdlib.h> /* free() */
 #include <string.h> /* strdup() */
 #include <time.h> /* time_t time() */
 
-#include "cfg/config.h"
 #include "utils/fs.h"
 #include "utils/macros.h"
+#include "utils/path.h"
 #include "utils/str.h"
 #include "filelist.h"
-#include "status.h"
 #include "ui.h"
 
+static int is_valid_index(const int bmark_index);
+static void clear_bmarks(bookmark_t bmarks[], int count);
 static void clear_mark(bookmark_t *bmark);
 static int is_user_bookmark(const char mark);
 static void set_mark(const char mark, const char directory[], const char file[],
@@ -41,19 +40,26 @@ static void set_mark(const char mark, const char directory[], const char file[],
 static int is_bmark_points_to(const bookmark_t *bmark, const char directory[],
 		const char file[]);
 static int navigate_to_bookmark(FileView *view, const char mark);
-static bookmark_t * get_bookmark(const char mark);
+TSTATIC bookmark_t * get_bmark_by_name(const char mark);
+static bookmark_t * get_bmark(const int bmark_index);
 static int is_bmark_valid(const bookmark_t *bmark);
 static int is_bmark_empty(const bookmark_t *bmark);
 
-bookmark_t bookmarks[NUM_BOOKMARKS];
+/* Data of regular bookmarks. */
+static bookmark_t regular_bookmarks[NUM_REGULAR_BOOKMARKS];
+
+/* Data of special bookmarks. */
+static bookmark_t lspecial_bookmarks[NUM_SPECIAL_BOOKMARKS];
+static bookmark_t rspecial_bookmarks[NUM_SPECIAL_BOOKMARKS];
 
 const char valid_bookmarks[] =
 {
-	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '<', '>',
+	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
 	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
 	'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
 	'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
 	'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
+	'<', '>',
 	'\0'
 };
 ARRAY_GUARD(valid_bookmarks, NUM_BOOKMARKS + 1);
@@ -65,29 +71,44 @@ static const char spec_bookmarks[] =
 	'<', '>', '\'',
 	'\0'
 };
+ARRAY_GUARD(spec_bookmarks, NUM_SPECIAL_BOOKMARKS + 1);
+
+const bookmark_t *
+get_bookmark(const int bmark_index)
+{
+	return get_bmark(bmark_index);
+}
 
 char
 index2mark(const int bmark_index)
 {
-	if(bmark_index >= 0 && bmark_index < ARRAY_LEN(valid_bookmarks) - 1)
+	if(is_valid_index(bmark_index))
 	{
 		return valid_bookmarks[bmark_index];
 	}
 	return '\0';
 }
 
+/* Checks whether bookmark index is valid.  Returns non-zero if so, otherwise
+ * zero is returned. */
+static int
+is_valid_index(const int bmark_index)
+{
+	return bmark_index >= 0 && bmark_index < ARRAY_LEN(valid_bookmarks) - 1;
+}
+
 int
 is_valid_bookmark(const int bmark_index)
 {
 	const char mark = index2mark(bmark_index);
-	const bookmark_t *const bmark = get_bookmark(mark);
+	const bookmark_t *const bmark = get_bmark_by_name(mark);
 	return is_bmark_valid(bmark);
 }
 
 int
 is_bookmark_empty(const char mark)
 {
-	const bookmark_t *const bmark = get_bookmark(mark);
+	const bookmark_t *const bmark = get_bmark_by_name(mark);
 	return is_bmark_empty(bmark);
 }
 
@@ -101,19 +122,25 @@ is_spec_bookmark(const int x)
 void
 clear_bookmark(const int mark)
 {
-	bookmark_t *const bmark = get_bookmark(mark);
+	bookmark_t *const bmark = get_bmark_by_name(mark);
 	clear_mark(bmark);
 }
 
 void
 clear_all_bookmarks(void)
 {
-	bookmark_t *bmark = &bookmarks[0];
-	const bookmark_t *const end = &bookmarks[ARRAY_LEN(bookmarks)];
-	while(bmark != end)
+	clear_bmarks(regular_bookmarks, ARRAY_LEN(regular_bookmarks));
+	clear_bmarks(lspecial_bookmarks, ARRAY_LEN(lspecial_bookmarks));
+	clear_bmarks(rspecial_bookmarks, ARRAY_LEN(rspecial_bookmarks));
+}
+
+static void
+clear_bmarks(bookmark_t bmarks[], int count)
+{
+	int i;
+	for(i = 0; i < count; ++i)
 	{
-		clear_mark(bmark);
-		bmark++;
+		clear_mark(&bmarks[i]);
 	}
 }
 
@@ -137,7 +164,7 @@ clear_mark(bookmark_t *bmark)
 int
 is_bookmark_older(const char mark, const time_t than)
 {
-	const bookmark_t *const bmark = get_bookmark(mark);
+	const bookmark_t *const bmark = get_bmark_by_name(mark);
 	if(bmark != NULL)
 	{
 		static const time_t undef_time = (time_t)-1;
@@ -201,7 +228,7 @@ static void
 set_mark(const char mark, const char directory[], const char file[],
 		time_t timestamp, int force)
 {
-	bookmark_t *const bmark = get_bookmark(mark);
+	bookmark_t *const bmark = get_bmark_by_name(mark);
 	if(bmark != NULL && (force || !is_bmark_points_to(bmark, directory, file)))
 	{
 		clear_mark(bmark);
@@ -227,11 +254,11 @@ is_bmark_points_to(const bookmark_t *bmark, const char directory[],
 int
 check_mark_directory(FileView *view, char mark)
 {
-	const bookmark_t *const bmark = get_bookmark(mark);
+	const bookmark_t *const bmark = get_bmark_by_name(mark);
 
 	if(!is_bmark_empty(bmark))
 	{
-		if(stroscmp(view->curr_dir, bmark->directory) == 0)
+		if(paths_are_equal(view->curr_dir, bmark->directory))
 		{
 			return find_file_pos_in_list(view, bmark->file);
 		}
@@ -263,13 +290,21 @@ goto_bookmark(FileView *view, char mark)
 static int
 navigate_to_bookmark(FileView *view, char mark)
 {
-	const bookmark_t *const bmark = get_bookmark(mark);
+	const bookmark_t *const bmark = get_bmark_by_name(mark);
 
 	if(is_bmark_valid(bmark))
 	{
-		if(change_directory(view, bmark->directory) >= 0)
+		/* Do not change directory if we already there. */
+		if(!paths_are_equal(view->curr_dir, bmark->directory))
+		{
+			if(change_directory(view, bmark->directory) >= 0)
+			{
+				load_dir_list(view, 1);
+			}
+		}
+
+		if(paths_are_equal(view->curr_dir, bmark->directory))
 		{
-			load_dir_list(view, 1);
 			(void)ensure_file_is_selected(view, bmark->file);
 		}
 	}
@@ -290,11 +325,34 @@ navigate_to_bookmark(FileView *view, char mark)
 
 /* Gets bookmark data structure by name of a bookmark.  Returns pointer to
  * bookmark's data structure or NULL. */
-static bookmark_t *
-get_bookmark(const char mark)
+TSTATIC bookmark_t *
+get_bmark_by_name(const char mark)
 {
-	const char *pos = strchr(valid_bookmarks, mark);
-	return (pos == NULL) ? NULL : &bookmarks[pos - valid_bookmarks];
+	const char *const pos = strchr(valid_bookmarks, mark);
+	return (pos == NULL) ? NULL : get_bmark(pos - valid_bookmarks);
+}
+
+/* Gets bookmark by its index.  Returns pointer to a statically allocated
+ * bookmark_t structure or NULL for wrong index. */
+static bookmark_t *
+get_bmark(const int bmark_index)
+{
+	int spec_bmark_index;
+
+	if(!is_valid_index(bmark_index))
+	{
+		return NULL;
+	}
+
+	if(bmark_index < NUM_REGULAR_BOOKMARKS)
+	{
+		return &regular_bookmarks[bmark_index];
+	}
+
+	spec_bmark_index = bmark_index - NUM_REGULAR_BOOKMARKS;
+	return (curr_view == &lwin)
+	     ? &lspecial_bookmarks[spec_bmark_index]
+	     : &rspecial_bookmarks[spec_bmark_index];
 }
 
 /* Checks if a bookmark is valid (exists and points to an existing directory).
@@ -316,7 +374,7 @@ init_active_bookmarks(const char marks[], int active_bookmarks[])
 	{
 		if(!char_is_one_of(marks, index2mark(x)))
 			continue;
-		if(is_bmark_empty(&bookmarks[x]))
+		if(is_bmark_empty(get_bookmark(x)))
 			continue;
 		active_bookmarks[i++] = x;
 	}
