@@ -19,7 +19,7 @@
 #include "cmds.h"
 
 #include <assert.h> /* assert() */
-#include <ctype.h>
+#include <ctype.h> /* isalpha() isdigit() isspace() */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h>
 #include <stdlib.h> /* calloc() malloc() free() realloc() */
@@ -30,6 +30,7 @@
 #include "../utils/str.h"
 #include "../utils/string_array.h"
 #include "../utils/test_helpers.h"
+#include "../utils/utils.h"
 #include "completion.h"
 
 #define MAX_CMD_RECURSION 16
@@ -81,18 +82,19 @@ static const char *RANGE_SEPARATORS = ",;";
 static inner_t *inner;
 static cmds_conf_t *cmds_conf;
 
-static const char * parse_limit(const char cmd[], cmd_info_t *cmd_info);
 static const char * correct_limit(const char cmd[], cmd_info_t *cmd_info);
 static int udf_is_ambiguous(const char name[]);
 static const char * parse_tail(cmd_t *cur, const char cmd[],
 		cmd_info_t *cmd_info);
-static const char *get_cmd_name(const char cmd[], char buf[], size_t buf_len);
+static const char * get_cmd_name(const char cmd[], char buf[], size_t buf_len);
 static void init_cmd_info(cmd_info_t *cmd_info);
 static const char * skip_prefix_commands(const char cmd[]);
 static cmd_t * find_cmd(const char name[]);
 static const char * parse_range(const char cmd[], cmd_info_t *cmd_info);
+static const char * parse_range_elem(const char cmd[], cmd_info_t *cmd_info,
+		char last_sep);
 static int complete_cmd_args(cmd_t *cur, const char args[],
-		cmd_info_t *cmd_info);
+		cmd_info_t *cmd_info, void *arg);
 static void complete_cmd_name(const char cmd_name[], int user_only);
 TSTATIC int add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf);
 static int comclear_cmd(const cmd_info_t *cmd_info);
@@ -104,9 +106,8 @@ static cmd_t * insert_cmd(cmd_t *after);
 static int delcommand_cmd(const cmd_info_t *cmd_info);
 TSTATIC char ** dispatch_line(const char args[], int *count, char sep,
 		int regexp, int quotes, int *last_arg, int *last_begin, int *last_end);
+static int is_separator(char c, char sep);
 TSTATIC void unescape(char s[], int regexp);
-static void replace_double_squotes(char s[]);
-static void replace_esc(char s[]);
 
 void
 init_cmds(int udf, cmds_conf_t *conf)
@@ -178,6 +179,8 @@ execute_cmd(const char cmd[])
 	cmd_t *cur;
 	const char *args;
 	int execution_code;
+	size_t last_arg_len;
+	char *last_arg;
 	int last_end;
 	cmds_conf_t *cc = cmds_conf;
 
@@ -217,15 +220,14 @@ execute_cmd(const char cmd[])
 	args = parse_tail(cur, cmd, &cmd_info);
 
 	cmd_info.raw_args = strdup(args);
-	if(cur->bg && ends_with(cmd_info.raw_args, " &"))
+
+	/* Set background flag and remove background mark from raw arguments, when
+	 * command supports backgrounding. */
+	last_arg = get_last_argument(cmd_info.raw_args, &last_arg_len);
+	if(cur->bg && *last_arg == '&' && *skip_whitespace(last_arg + 1) == '\0')
 	{
 		cmd_info.bg = 1;
-		cmd_info.raw_args[strlen(cmd_info.raw_args) - 2] = '\0';
-	}
-	else if(cur->bg && strcmp(cmd_info.raw_args, "&") == 0)
-	{
-		cmd_info.bg = 1;
-		cmd_info.raw_args[0] = '\0';
+		*last_arg = '\0';
 	}
 
 	if(cur->select)
@@ -309,65 +311,6 @@ execute_cmd(const char cmd[])
 	return execution_code;
 }
 
-static const char *
-parse_limit(const char cmd[], cmd_info_t *cmd_info)
-{
-	if(cmd[0] == '%')
-	{
-		cmd_info->begin = cmds_conf->begin;
-		cmd_info->end = cmds_conf->end;
-		cmd++;
-	}
-	else if(cmd[0] == '$')
-	{
-		cmd_info->end = cmds_conf->end;
-		cmd++;
-	}
-	else if(cmd[0] == '.')
-	{
-		cmd_info->end = cmds_conf->current;
-		cmd++;
-	}
-	else if(char_is_one_of(RANGE_SEPARATORS, *cmd))
-	{
-		cmd_info->end = cmds_conf->current;
-	}
-	else if(isalpha(*cmd))
-	{
-		cmd_info->end = cmds_conf->current;
-	}
-	else if(isdigit(*cmd))
-	{
-		char *p;
-		cmd_info->end = strtol(cmd, &p, 10) - 1;
-		if(cmd_info->end < cmds_conf->begin)
-			cmd_info->end = cmds_conf->begin;
-		cmd = p;
-	}
-	else if(*cmd == '\'')
-	{
-		char mark;
-		cmd++;
-		mark = *cmd++;
-		cmd_info->end = cmds_conf->resolve_mark(mark);
-		if(cmd_info->end < 0)
-		{
-			cmd_info->end = INVALID_MARK;
-			return NULL;
-		}
-	}
-	else if(*cmd == '+' || *cmd == '-')
-	{
-		cmd_info->end = cmds_conf->current;
-	}
-	else
-	{
-		return NULL;
-	}
-
-	return cmd;
-}
-
 static const char *
 correct_limit(const char cmd[], cmd_info_t *cmd_info)
 {
@@ -444,14 +387,21 @@ parse_tail(cmd_t *cur, const char cmd[], cmd_info_t *cmd_info)
 		cmd_info->qmark = 1;
 		cmd++;
 	}
+
 	if(*cmd != '\0' && !isspace(*cmd))
 	{
 		if(cur->cust_sep)
-			cmd_info->sep = *cmd;
+		{
+			cmd_info->sep = isspace(*cmd) ? ' ' : *cmd;
+		}
 		return cmd;
 	}
-	while(*cmd == cmd_info->sep)
-		cmd++;
+
+	while(is_separator(*cmd, cmd_info->sep))
+	{
+		++cmd;
+	}
+
 	return cmd;
 }
 
@@ -512,7 +462,7 @@ get_cmd_info(const char cmd[], cmd_info_t *info)
 }
 
 int
-complete_cmd(const char cmd[])
+complete_cmd(const char cmd[], void *arg)
 {
 	cmd_info_t cmd_info;
 	const char *begin, *cmd_name_pos;
@@ -534,7 +484,7 @@ complete_cmd(const char cmd[])
 		args = get_cmd_name(cmd_name_pos, cmd_name, sizeof(cmd_name));
 		cur = find_cmd(cmd_name);
 
-		if(*args == '\0' && *args != '!' && strcmp(cmd_name, "!") != 0)
+		if(*args == '\0' && strcmp(cmd_name, "!") != 0)
 		{
 			complete_cmd_name(cmd_name, 0);
 			prefix_len += cmd_name_pos - cmd;
@@ -542,7 +492,7 @@ complete_cmd(const char cmd[])
 		else
 		{
 			prefix_len += args - cmd;
-			prefix_len += complete_cmd_args(cur, args, &cmd_info);
+			prefix_len += complete_cmd_args(cur, args, &cmd_info, arg);
 		}
 	}
 
@@ -612,20 +562,27 @@ find_cmd(const char name[])
 	return cmd;
 }
 
-/* Returns NULL on invalid range. */
+/* Parses whole command range (e.g. "<val>;+<val>,,-<val>").  Returns advanced
+ * value of cmd when parsing is successful, otherwise NULL is returned. */
 static const char *
 parse_range(const char cmd[], cmd_info_t *cmd_info)
 {
+	char last_sep;
+
 	cmd = skip_whitespace(cmd);
 
 	if(isalpha(*cmd) || *cmd == '!' || *cmd == '\0')
+	{
 		return cmd;
+	}
 
+	last_sep = '\0';
 	while(*cmd != '\0')
 	{
 		cmd_info->begin = cmd_info->end;
 
-		if((cmd = parse_limit(cmd, cmd_info)) == NULL)
+		cmd = parse_range_elem(cmd, cmd_info, last_sep);
+		if(cmd == NULL)
 		{
 			return NULL;
 		}
@@ -642,6 +599,7 @@ parse_range(const char cmd[], cmd_info_t *cmd_info)
 			break;
 		}
 
+		last_sep = *cmd;
 		cmd++;
 
 		cmd = skip_whitespace(cmd);
@@ -650,6 +608,73 @@ parse_range(const char cmd[], cmd_info_t *cmd_info)
 	return cmd;
 }
 
+/* Parses single element of a command range (e.g. any of <el> in
+ * "<el>;<el>,<el>").  Returns advanced value of cmd when parsing is successful,
+ * otherwise NULL is returned. */
+static const char *
+parse_range_elem(const char cmd[], cmd_info_t *cmd_info, char last_sep)
+{
+	if(cmd[0] == '%')
+	{
+		cmd_info->begin = cmds_conf->begin;
+		cmd_info->end = cmds_conf->end;
+		cmd++;
+	}
+	else if(cmd[0] == '$')
+	{
+		cmd_info->end = cmds_conf->end;
+		cmd++;
+	}
+	else if(cmd[0] == '.')
+	{
+		cmd_info->end = cmds_conf->current;
+		cmd++;
+	}
+	else if(char_is_one_of(RANGE_SEPARATORS, *cmd))
+	{
+		cmd_info->end = cmds_conf->current;
+	}
+	else if(isalpha(*cmd))
+	{
+		cmd_info->end = cmds_conf->current;
+	}
+	else if(isdigit(*cmd))
+	{
+		char *p;
+		cmd_info->end = strtol(cmd, &p, 10) - 1;
+		if(cmd_info->end < cmds_conf->begin)
+			cmd_info->end = cmds_conf->begin;
+		cmd = p;
+	}
+	else if(*cmd == '\'')
+	{
+		char mark;
+		cmd++;
+		mark = *cmd++;
+		cmd_info->end = cmds_conf->resolve_mark(mark);
+		if(cmd_info->end < 0)
+		{
+			cmd_info->end = INVALID_MARK;
+			return NULL;
+		}
+	}
+	else if(*cmd == '+' || *cmd == '-')
+	{
+		/* Do nothing after semicolon, because in this case +/- are adjusting not
+		 * base current cursor position, but base end of the range. */
+		if(last_sep != ';')
+		{
+			cmd_info->end = cmds_conf->current;
+		}
+	}
+	else
+	{
+		return NULL;
+	}
+
+	return cmd;
+}
+
 static const char *
 get_cmd_name(const char cmd[], char buf[], size_t buf_len)
 {
@@ -697,7 +722,8 @@ get_cmd_name(const char cmd[], char buf[], size_t buf_len)
 
 /* Returns offset at which completion was done. */
 static int
-complete_cmd_args(cmd_t *cur, const char args[], cmd_info_t *cmd_info)
+complete_cmd_args(cmd_t *cur, const char args[], cmd_info_t *cmd_info,
+		void *arg)
 {
 	const char *tmp_args = args;
 	int result = 0;
@@ -726,7 +752,8 @@ complete_cmd_args(cmd_t *cur, const char args[], cmd_info_t *cmd_info)
 		int last_arg = 0;
 
 		argv = dispatch_line(args, &argc, ' ', 0, 1, &last_arg, NULL, NULL);
-		result += cmds_conf->complete_args(cur->id, args, argc, argv, last_arg);
+		result += cmds_conf->complete_args(cur->id, args, argc, argv, last_arg,
+				arg);
 		free_string_array(argv, argc);
 	}
 	return result;
@@ -752,11 +779,11 @@ complete_cmd_name(const char cmd_name[], int user_only)
 		else if(cur->name[0] == '\0')
 			;
 		else
-			add_completion(cur->name);
+			vle_compl_add_match(cur->name);
 		cur = cur->next;
 	}
 
-	add_completion(cmd_name);
+	vle_compl_add_last_match(cmd_name);
 }
 
 void
@@ -775,7 +802,8 @@ add_builtin_commands(const cmd_add_t *cmds, int count)
 		{
 			size_t full_len, short_len;
 			char buf[strlen(cmds[i].name) + 1];
-			assert(starts_with(cmds[i].name, cmds[i].abbr));
+			assert(starts_with(cmds[i].name, cmds[i].abbr) &&
+					"Abbreviation is consistent with full command");
 			strcpy(buf, cmds[i].name);
 			full_len = strlen(buf);
 			short_len = strlen(cmds[i].abbr);
@@ -1059,13 +1087,14 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 
 	args_beg = args;
 	if(sep == ' ')
-		while(args[0] == sep)
-			args++;
+	{
+		args = skip_whitespace(args);
+	}
 	cmdstr = strdup(args);
 	len = strlen(cmdstr);
 	for(i = 0, st = 0, state = BEGIN; i <= len; ++i)
 	{
-		int prev_state = state;
+		const int prev_state = state;
 		switch(state)
 		{
 			case BEGIN:
@@ -1084,12 +1113,12 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 					st = i + 1;
 					state = R_QUOTING;
 				}
-				else if(cmdstr[i] != sep)
+				else if(!is_separator(cmdstr[i], sep))
 				{
 					st = i;
 					state = NO_QUOTING;
 				}
-				else if(sep != ' ' && i > 0 && cmdstr[i - 1] == sep)
+				else if(sep != ' ' && i > 0 && is_separator(cmdstr[i - 1], sep))
 				{
 					st = i--;
 					state = NO_QUOTING;
@@ -1100,7 +1129,7 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 				}
 				break;
 			case NO_QUOTING:
-				if(cmdstr[i] == '\0' || cmdstr[i] == sep)
+				if(cmdstr[i] == '\0' || is_separator(cmdstr[i], sep))
 				{
 					state = ARG;
 				}
@@ -1162,14 +1191,13 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 			last_arg = params[*count - 1];
 
 			cmdstr[i] = c;
-			if(prev_state == NO_QUOTING)
-				unescape(last_arg, (sep == ' ') ? 0 : 1);
-			else if(prev_state == S_QUOTING)
-				replace_double_squotes(last_arg);
-			else if(prev_state == D_QUOTING)
-				replace_esc(last_arg);
-			else if(prev_state == R_QUOTING)
-				unescape(last_arg, 1);
+			switch(prev_state)
+			{
+				case NO_QUOTING: unescape(last_arg, sep != ' ');    break;
+				case  S_QUOTING: expand_squotes_escaping(last_arg); break;
+				case  D_QUOTING: expand_dquotes_escaping(last_arg); break;
+				case  R_QUOTING: unescape(last_arg, 1);             break;
+			}
 			state = BEGIN;
 		}
 	}
@@ -1192,6 +1220,16 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 	return params;
 }
 
+/* Checks whether character is command separator.  Special case is space as a
+ * separator, which means that any whitespace can be used as separators.
+ * Returns non-zero if c is counted to be a separator, otherwise zero is
+ * returned. */
+static int
+is_separator(char c, char sep)
+{
+	return (sep == ' ') ? isspace(c) : (c == sep);
+}
+
 TSTATIC void
 unescape(char s[], int regexp)
 {
@@ -1211,74 +1249,6 @@ unescape(char s[], int regexp)
 	*p = '\0';
 }
 
-/* Replaces all '' with ' in place. */
-static void
-replace_double_squotes(char s[])
-{
-	char *p;
-	int sq_found;
-
-	p = s++;
-	sq_found = *p == '\'';
-	while(*p != '\0')
-	{
-		if(*s == '\'' && sq_found)
-		{
-			sq_found = 0;
-		}
-		else
-		{
-			*++p = *s;
-			sq_found = *s == '\'';
-		}
-		s++;
-	}
-}
-
-static void
-replace_esc(char s[])
-{
-	static const char table[] =
-						/* 00  01  02  03  04  05  06  07  08  09  0a  0b  0c  0d  0e  0f */
-	/* 00 */	"\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
-	/* 10 */	"\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f"
-	/* 20 */	"\x20\x21\x22\x23\x24\x25\x26\x27\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f"
-	/* 30 */	"\x00\x31\x32\x33\x34\x35\x36\x37\x38\x39\x3a\x3b\x3c\x3d\x3e\x3f"
-	/* 40 */	"\x40\x41\x42\x43\x44\x45\x46\x47\x48\x49\x4a\x4b\x4c\x4d\x4e\x4f"
-	/* 50 */	"\x50\x51\x52\x53\x54\x55\x56\x57\x58\x59\x5a\x5b\x5c\x5d\x5e\x5f"
-	/* 60 */	"\x60\x07\x0b\x63\x64\x65\x0c\x67\x68\x69\x6a\x6b\x6c\x6d\x0a\x6f"
-	/* 70 */	"\x70\x71\x0d\x73\x09\x75\x0b\x77\x78\x79\x7a\x7b\x7c\x7d\x7e\x7f"
-	/* 80 */	"\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f"
-	/* 90 */	"\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f"
-	/* a0 */	"\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf"
-	/* b0 */	"\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf"
-	/* c0 */	"\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf"
-	/* d0 */	"\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf"
-	/* e0 */	"\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef"
-	/* f0 */	"\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff";
-
-	char *str = s;
-	char *p;
-
-	p = s;
-	while(*s != '\0')
-	{
-		if(*s != '\\')
-		{
-			*p++ = *s++;
-			continue;
-		}
-		s++;
-		if(*s == '\0')
-		{
-			LOG_ERROR_MSG("Escaped eol in \"%s\"", str);
-			break;
-		}
-		*p++ = table[(int)*s++];
-	}
-	*p = '\0';
-}
-
 char **
 list_udf(void)
 {
