@@ -37,6 +37,7 @@
 #include "filename_modifiers.h"
 #include "registers.h"
 #include "status.h"
+#include "ui.h"
 
 TSTATIC char * append_selected_files(FileView *view, char expanded[],
 		int under_cursor, int quotes, const char mod[], int for_shell);
@@ -237,9 +238,6 @@ expand_macros(const char *command, const char *args, MacroFlags *flags,
 	}
 	while(x < cmd_len);
 
-	if(len > cfg.max_args/2)
-		show_error_msg("Argument is too long", " FIXME ");
-
 	return expanded;
 }
 
