@@ -25,7 +25,7 @@
 #include "colorscheme_menu.h"
 #include "commands_menu.h"
 #include "dirhistory_menu.h"
-#include "dirstask_menu.h"
+#include "dirstack_menu.h"
 #include "filetypes_menu.h"
 #include "find_menu.h"
 #include "grep_menu.h"
@@ -34,6 +34,8 @@
 #include "locate_menu.h"
 #include "map_menu.h"
 #include "registers_menu.h"
+#include "trash_menu.h"
+#include "trashes_menu.h"
 #include "undolist_menu.h"
 #include "users_menu.h"
 #include "vifm_menu.h"
