@@ -30,8 +30,11 @@
 #include "../macros.h"
 #include "../running.h"
 #include "../status.h"
+#include "../ui.h"
 #include "menus.h"
 
+static int execute_apropos_cb(FileView *view, menu_info *m);
+
 int
 show_apropos_menu(FileView *view, const char args[])
 {
@@ -39,13 +42,14 @@ show_apropos_menu(FileView *view, const char args[])
 	int save_msg;
 	custom_macro_t macros[] =
 	{
-		{ .letter = 'a', .value = args, .uses_left = 1 },
+		{ .letter = 'a', .value = args, .uses_left = 1, .group = -1 },
 	};
 
 	static menu_info m;
-	init_menu_info(&m, APROPOS, format_str("No matches for \'%s\'", args));
+	init_menu_info(&m, APROPOS_MENU, format_str("No matches for \'%s\'", args));
 	m.args = strdup(args);
 	m.title = format_str(" Apropos %s ", args);
+	m.execute_handler = &execute_apropos_cb;
 
 	status_bar_message("apropos...");
 
@@ -55,8 +59,10 @@ show_apropos_menu(FileView *view, const char args[])
 	return save_msg;
 }
 
-void
-execute_apropos_cb(menu_info *m)
+/* Callback that is called when menu item is selected.  Should return non-zero
+ * to stay in menu mode. */
+static int
+execute_apropos_cb(FileView *view, menu_info *m)
 {
 	char *line;
 	char *man_page;
@@ -68,7 +74,7 @@ execute_apropos_cb(menu_info *m)
 	if(free_this == NULL)
 	{
 		show_error_msg("Memory Error", "Unable to allocate enough memory");
-		return;
+		return 1;
 	}
 
 	if((num_str = strchr(line, '(')))
@@ -82,7 +88,7 @@ execute_apropos_cb(menu_info *m)
 			if(z > 40)
 			{
 				free(free_this);
-				return;
+				return 1;
 			}
 		}
 
@@ -94,12 +100,13 @@ execute_apropos_cb(menu_info *m)
 
 			snprintf(command, sizeof(command), "man %s %s", num_str, man_page);
 
-			curr_stats.auto_redraws = 1;
+			curr_stats.skip_shellout_redraw = 1;
 			shellout(command, 0, 1);
-			curr_stats.auto_redraws = 0;
+			curr_stats.skip_shellout_redraw = 0;
 		}
 	}
 	free(free_this);
+	return 1;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
