@@ -36,6 +36,8 @@
 #define DEFAULT_PREDICATE "-name"
 #endif
 
+static int execute_find_cb(FileView *view, menu_info *m);
+
 int
 show_find_menu(FileView *view, int with_path, const char args[])
 {
@@ -46,24 +48,29 @@ show_find_menu(FileView *view, int with_path, const char args[])
 
 	custom_macro_t macros[] =
 	{
-		{ .letter = 's', .value = NULL, .uses_left = 1 },
-		{ .letter = 'a', .value = NULL, .uses_left = 1 },
+		{ .letter = 's', .value = NULL, .uses_left = 1, .group = -1 },
+		{ .letter = 'a', .value = NULL, .uses_left = 1, .group =  1 },
+		{ .letter = 'A', .value = NULL, .uses_left = 0, .group =  1 },
 	};
 
 	static menu_info m;
-	init_menu_info(&m, FIND, strdup("No files found"));
+	init_menu_info(&m, FIND_MENU, strdup("No files found"));
 
 	m.title = format_str(" Find %s ", args);
+	m.execute_handler = &execute_find_cb;
+	m.key_handler = &filelist_khandler;
 
 	if(with_path)
 	{
 		macros[0].value = args;
 		macros[1].value = "";
+		macros[2].value = "";
 	}
 	else
 	{
 		targets = get_cmd_target();
 		macros[0].value = targets;
+		macros[2].value = args;
 
 		if(args[0] == '-')
 		{
@@ -91,5 +98,14 @@ show_find_menu(FileView *view, int with_path, const char args[])
 	return save_msg;
 }
 
+/* Callback that is called when menu item is selected.  Should return non-zero
+ * to stay in menu mode. */
+static int
+execute_find_cb(FileView *view, menu_info *m)
+{
+	goto_selected_file(view, m->items[m->pos], 0);
+	return 0;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
