@@ -30,6 +30,8 @@
 #include "../ui.h"
 #include "menus.h"
 
+static int execute_grep_cb(FileView *view, menu_info *m);
+
 int
 show_grep_menu(FileView *view, const char args[], int invert)
 {
@@ -40,20 +42,24 @@ show_grep_menu(FileView *view, const char args[], int invert)
 
 	custom_macro_t macros[] =
 	{
-		{ .letter = 'i', .value = NULL, .uses_left = 1 },
-		{ .letter = 'a', .value = NULL, .uses_left = 1 },
-		{ .letter = 's', .value = NULL, .uses_left = 1 },
+		{ .letter = 'i', .value = NULL, .uses_left = 1, .group = -1 },
+		{ .letter = 'a', .value = NULL, .uses_left = 1, .group =  1 },
+		{ .letter = 's', .value = NULL, .uses_left = 1, .group = -1 },
+		{ .letter = 'A', .value = NULL, .uses_left = 0, .group =  1 },
 	};
 
 	static menu_info m;
-	init_menu_info(&m, GREP, strdup("No matches found"));
+	init_menu_info(&m, GREP_MENU, strdup("No matches found"));
 
 	m.title = format_str(" Grep %s ", args);
+	m.execute_handler = &execute_grep_cb;
+	m.key_handler = &filelist_khandler;
 
 	targets = get_cmd_target();
 	macros[0].value = invert ? "-v" : "";
 	macros[1].value = args;
 	macros[2].value = targets;
+	macros[3].value = args;
 	if(args[0] != '-')
 	{
 		escaped_args = escape_filename(args, 0);
@@ -72,5 +78,14 @@ show_grep_menu(FileView *view, const char args[], int invert)
 	return save_msg;
 }
 
+/* Callback that is called when menu item is selected.  Should return non-zero
+ * to stay in menu mode. */
+static int
+execute_grep_cb(FileView *view, menu_info *m)
+{
+	goto_selected_file(view, m->items[m->pos], 1);
+	return 1;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
