@@ -19,10 +19,10 @@
 
 #include "undolist_menu.h"
 
+#include <stddef.h> /* size_t */
 #include <stdlib.h> /* realloc() */
 #include <string.h> /* strdup() strlen() */
 
-#include "../modes/menu.h"
 #include "../utils/string_array.h"
 #include "../ui.h"
 #include "../undo.h"
