@@ -19,7 +19,12 @@
 
 #include "attr_dialog_nix.h"
 
+#include <curses.h>
+
 #include <fcntl.h>
+#include <sys/stat.h> /* S_* */
+#include <sys/types.h> /* mode_t */
+#include <unistd.h> /* uid_t geteuid() */
 
 #include <assert.h> /* assert() */
 #include <stddef.h> /* NULL size_t */
@@ -27,14 +32,14 @@
 #include <string.h> /* strncat() strlen() */
 
 #include "../../engine/keys.h"
+#include "../../engine/mode.h"
 #include "../../menus/menus.h"
 #include "../../utils/fs.h"
 #include "../../utils/fs_limits.h"
 #include "../../utils/macros.h"
 #include "../../utils/path.h"
-#include "../../background.h"
 #include "../../filelist.h"
-#include "../../fileops.h"
+#include "../../ops.h"
 #include "../../status.h"
 #include "../../ui.h"
 #include "../../undo.h"
@@ -61,7 +66,6 @@ static void cmd_k(key_info_t key_info, keys_info_t *keys_info);
 static void inc_curr(void);
 static void dec_curr(void);
 
-static int *mode;
 static FileView *view;
 static int top, bottom;
 static int curr;
@@ -103,16 +107,14 @@ static keys_add_info_t builtin_cmds[] = {
 };
 
 void
-init_attr_dialog_mode(int *key_mode)
+init_attr_dialog_mode(void)
 {
 	int ret_code;
 
-	assert(key_mode != NULL);
-
-	mode = key_mode;
-
 	ret_code = add_cmds(builtin_cmds, ARRAY_LEN(builtin_cmds), ATTR_MODE);
 	assert(ret_code == 0);
+
+	(void)ret_code;
 }
 
 void
@@ -167,7 +169,7 @@ enter_attr_mode(FileView *active_view)
 		i++;
 	}
 
-	*mode = ATTR_MODE;
+	vle_mode_set(ATTR_MODE, VMT_SECONDARY);
 	clear_input_bar();
 	curr_stats.use_input_bar = 0;
 
@@ -371,7 +373,7 @@ get_selection_size(int first_file_index)
 static void
 leave_attr_mode(void)
 {
-	*mode = NORMAL_MODE;
+	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
 	curs_set(FALSE);
 	curr_stats.use_input_bar = 1;
 
@@ -542,8 +544,10 @@ file_chmod(char *path, const char *mode, const char *inv_mode, int recurse_dirs)
 {
 	int op = recurse_dirs ? OP_CHMODR : OP_CHMOD;
 
-	if(perform_operation(op, (void *)mode, path, NULL) == 0)
+	if(perform_operation(op, NULL, (void *)mode, path, NULL) == 0)
+	{
 		add_operation(op, strdup(mode), strdup(inv_mode), path, "");
+	}
 }
 
 static void
