@@ -22,9 +22,13 @@
 
 #include "../../ui.h"
 
-void init_attr_dialog_mode(int *key_mode);
+/* Initializes attributes dialog mode. */
+void init_attr_dialog_mode(void);
+
 void enter_attr_mode(FileView *active_view);
+
 void redraw_attr_dialog(void);
+
 void files_chmod(FileView *view, const char *mode, int recurse_dirs);
 
 #endif /* VIFM__MODES__DIALOGS__ATTR_DIALOG_NIX_H__ */
