@@ -21,8 +21,11 @@
 
 #include "../../ui.h"
 
-void init_attr_dialog_mode(int *key_mode);
+/* Initializes attributes dialog mode. */
+void init_attr_dialog_mode(void);
+
 void enter_attr_mode(FileView *active_view);
+
 void redraw_attr_dialog(void);
 
 #endif /* VIFM__MODES__DIALOGS__ATTR_DIALOG_WIN_H__ */
