@@ -22,13 +22,33 @@
 
 #include "../ui.h"
 
-void init_visual_mode(int *key_mode);
-void enter_visual_mode(int restore_selection);
+/* Generic submodes of the visual mode. */
+typedef enum
+{
+	VS_NORMAL,  /* With initial clearing of selection. */
+	VS_RESTORE, /* With restoring of previous selection if available. */
+	VS_AMEND,   /* Amending existing selection. */
+}
+VisualSubmodes;
+
+/* Initiailizes visual mode. */
+void init_visual_mode(void);
+
+/* Starts visual mode in a given submode. */
+void enter_visual_mode(VisualSubmodes sub_mode);
+
 void leave_visual_mode(int save_msg, int goto_top, int clean_selection);
+
 /* Should be used to ask visual mode to redraw file list correctly.
- * Intented to be used after setting list position from side. */
+ * Intended to be used after setting list position from side. */
 void update_visual_mode(void);
-int find_vpattern(FileView *view, const char *pattern, int backward);
+
+int find_vpattern(FileView *view, const char *pattern, int backward,
+		int interactive);
+
+/* Formats concise description of current visual mode state.  Returns pointer
+ * to a statically allocated buffer. */
+const char * describe_visual_mode(void);
 
 #endif /* VIFM__MODES__VISUAL_H__ */
 
