@@ -23,8 +23,12 @@
 #include "ui.h"
 
 void quick_view_file(FileView * view);
+
 void toggle_quick_view(void);
 
+/* Quits preview pane or view modes. */
+void preview_close(void);
+
 #endif /* VIFM__QUICKVIEW_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
