@@ -23,6 +23,8 @@ const char *tags[] = {
 	"vifm-'",
 	"vifm-'aproposprg'",
 	"vifm-'autochpos'",
+	"vifm-'cd'",
+	"vifm-'cdpath'",
 	"vifm-'cf'",
 	"vifm-'classify'",
 	"vifm-'co'",
@@ -32,6 +34,8 @@ const char *tags[] = {
 	"vifm-'cpoptions'",
 	"vifm-'dotdirs'",
 	"vifm-'fastrun'",
+	"vifm-'fcs'",
+	"vifm-'fillchars'",
 	"vifm-'findprg'",
 	"vifm-'followlinks'",
 	"vifm-'fusehome'",
@@ -76,13 +80,16 @@ const char *tags[] = {
 	"vifm-'sortorder'",
 	"vifm-'statusline'",
 	"vifm-'stl'",
+	"vifm-'syscalls'",
 	"vifm-'tabstop'",
 	"vifm-'timefmt'",
 	"vifm-'timeoutlen'",
 	"vifm-'tm'",
+	"vifm-'to'",
 	"vifm-'trash'",
 	"vifm-'trashdir'",
 	"vifm-'ts'",
+	"vifm-'tuioptions",
 	"vifm-'ul'",
 	"vifm-'undolevels'",
 	"vifm-'vicmd'",
@@ -311,6 +318,7 @@ const char *tags[] = {
 	"vifm-CTRL-W_L",
 	"vifm-CTRL-W__",
 	"vifm-CTRL-W_b",
+	"vifm-CTRL-W_bar",
 	"vifm-CTRL-W_h",
 	"vifm-CTRL-W_j",
 	"vifm-CTRL-W_k",
@@ -350,6 +358,8 @@ const char *tags[] = {
 	"vifm-[count]",
 	"vifm-^",
 	"vifm-al",
+	"vifm-app.txt",
+	"vifm-av",
 	"vifm-cW",
 	"vifm-c_ALT-.",
 	"vifm-c_ALT-B",
@@ -402,8 +412,8 @@ const char *tags[] = {
 	"vifm-cl",
 	"vifm-clientserver",
 	"vifm-co",
+	"vifm-color-schemes",
 	"vifm-colors",
-	"vifm-colorschemes",
 	"vifm-column-view",
 	"vifm-command-line",
 	"vifm-command-line-edit",
@@ -420,6 +430,8 @@ const char *tags[] = {
 	"vifm-d",
 	"vifm-dd",
 	"vifm-e",
+	"vifm-env-vars",
+	"vifm-executable()",
 	"vifm-expand()",
 	"vifm-expr-!=",
 	"vifm-expr-'",
@@ -435,6 +447,7 @@ const char *tags[] = {
 	"vifm-expr-option",
 	"vifm-expr-quote",
 	"vifm-expr-string",
+	"vifm-expr-unary-!",
 	"vifm-expr-unary-+",
 	"vifm-expr-unary--",
 	"vifm-expr1",
@@ -467,6 +480,7 @@ const char *tags[] = {
 	"vifm-guu",
 	"vifm-gv",
 	"vifm-h",
+	"vifm-has()",
 	"vifm-i",
 	"vifm-j",
 	"vifm-k",
@@ -554,6 +568,7 @@ const char *tags[] = {
 	"vifm-q_CTRL-W_L",
 	"vifm-q_CTRL-W__",
 	"vifm-q_CTRL-W_b",
+	"vifm-q_CTRL-W_bar",
 	"vifm-q_CTRL-W_h",
 	"vifm-q_CTRL-W_j",
 	"vifm-q_CTRL-W_k",
@@ -602,16 +617,20 @@ const char *tags[] = {
 	"vifm-set-options",
 	"vifm-startup",
 	"vifm-t",
+	"vifm-to-p",
+	"vifm-to-s",
 	"vifm-trash",
 	"vifm-u",
 	"vifm-v",
 	"vifm-v_:",
 	"vifm-v_CTRL-C",
+	"vifm-v_CTRL-G",
 	"vifm-v_Enter",
 	"vifm-v_Escape",
 	"vifm-v_O",
 	"vifm-v_U",
 	"vifm-v_V",
+	"vifm-v_av",
 	"vifm-v_gU",
 	"vifm-v_gu",
 	"vifm-v_gv",
@@ -635,6 +654,5 @@ const char *tags[] = {
 	"vifm-zo",
 	"vifm-zt",
 	"vifm-zz",
-	"vifm.txt",
 	0,
 };
