@@ -19,14 +19,18 @@
 #ifndef VIFM__TRASH_H__
 #define VIFM__TRASH_H__
 
+/* Description of a single trash item. */
 typedef struct
 {
-	char *path;
-	char *trash_name;
+	char *path;       /* Original full path of file before its removal. */
+	char *trash_name; /* Full path of file inside trash directory. */
 }
 trash_entry_t;
 
+/* List of items in trashes. */
 trash_entry_t *trash_list;
+
+/* Number of items in the trash_list. */
 int nentries;
 
 /* Parses trash directory name specification.  Sets value of cfg.trash_dir as a
@@ -64,6 +68,11 @@ int remove_from_trash(const char trash_name[]);
  * trash directory available NULL is returned. */
 char * gen_trash_name(const char base_dir[], const char name[]);
 
+/* Picks trash directory basing on original directory of a file that is being
+ * trashed.  Returns absolute path to picked trash directory on success which
+ * should be freed by the caller, otherwise NULL is returned. */
+char * pick_trash_dir(const char base_dir[]);
+
 /* Checks whether given absolute path points to a file under trash directory.
  * Returns non-zero if so, otherwise zero is returned. */
 int is_under_trash(const char path[]);
@@ -76,6 +85,9 @@ int is_trash_directory(const char path[]);
  * Returns that pointer. */
 const char * get_real_name_from_trash_name(const char trash_path[]);
 
+/* Removes entries that correspond to nonexistent files in trashes. */
+void trash_prune_dead_entries(void);
+
 #endif /* VIFM__TRASH_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
