@@ -20,12 +20,15 @@
 #include "utils_nix.h"
 #include "utils_int.h"
 
+#include <sys/select.h> /* select() FD_SET FD_ZERO */
 #include <sys/stat.h> /* S_* */
+#include <sys/time.h> /* timeval */
 #include <sys/types.h> /* gid_t mode_t pid_t uid_t */
+#include <sys/wait.h> /* waitpid */
 #include <fcntl.h> /* O_RDONLY open() close() */
 #include <grp.h> /* getgrnam() */
 #include <pwd.h> /* getpwnam() */
-#include <unistd.h> /* dup2() getpid() */
+#include <unistd.h> /* X_OK access() dup2() getpid() */
 
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
@@ -35,12 +38,15 @@
                        SIG_UNBLOCK */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* snprintf() */
-#include <stdlib.h> /* atoi() */
-#include <string.h> /* strchr() strlen() strncmp() */
+#include <stdlib.h> /* atoi() free() */
+#include <string.h> /* strchr() strdup() strlen() strncmp() */
 
 #include "../cfg/config.h"
+#include "../ui.h"
+#include "fs.h"
 #include "fs_limits.h"
 #include "log.h"
+#include "macros.h"
 #include "mntent.h" /* mntent setmntent() getmntent() endmntent() */
 #include "path.h"
 #include "str.h"
@@ -65,9 +71,9 @@ typedef struct
 }
 get_mount_point_traverser_state;
 
-static void process_cancel_request(pid_t pid);
 static int get_mount_info_traverser(struct mntent *entry, void *arg);
-static int begins_with_list_item(const char pattern[], const char list[]);
+static int starts_with_list_item(const char str[], const char list[]);
+static int find_path_prefix_index(const char path[], const char list[]);
 
 void
 pause_shell(void)
@@ -137,6 +143,7 @@ wait_for_data_from(pid_t pid, FILE *f, int fd)
 {
 	const struct timeval ts_init = { .tv_sec = 0, .tv_usec = 1000 };
 	struct timeval ts;
+	int select_result;
 
 	fd_set read_ready;
 	FD_ZERO(&read_ready);
@@ -148,14 +155,9 @@ wait_for_data_from(pid_t pid, FILE *f, int fd)
 		process_cancel_request(pid);
 		ts = ts_init;
 		FD_SET(fd, &read_ready);
+		select_result = select(fd + 1, &read_ready, NULL, NULL, &ts);
 	}
-	while(select(fd + 1, &read_ready, NULL, NULL, &ts) == 0);
-
-	/* Inform other parts of the application that cancellation took place. */
-	if(errno == EINTR)
-	{
-		ui_cancellation_request();
-	}
+	while(select_result == 0 || (select_result == -1 && errno == EINTR));
 }
 
 int
@@ -169,9 +171,7 @@ set_sigchld(int block)
 	    || sigprocmask(action, &sigchld_mask, NULL) == -1;
 }
 
-/* Checks whether cancelling of current operation is requested and sends SIGINT
- * to process specified by its process id to request cancellation. */
-static void
+void
 process_cancel_request(pid_t pid)
 {
 	if(ui_cancellation_requested())
@@ -273,6 +273,21 @@ get_perm_string(char buf[], int len, mode_t mode)
 		buf[3] = (buf[3] == '-') ? 'S' : 's';
 }
 
+int
+refers_to_slower_fs(const char from[], const char to[])
+{
+	const int i = find_path_prefix_index(to, cfg.slow_fs_list);
+	/* When destination is not on slow file system, no performance penalties are
+	 * expected. */
+	if(i == -1)
+	{
+		return 0;
+	}
+
+	/* Otherwise, same slowdown as we have for the source location is bearable. */
+	return find_path_prefix_index(from, cfg.slow_fs_list) != i;
+}
+
 int
 is_on_slow_fs(const char full_path[])
 {
@@ -296,10 +311,14 @@ is_on_slow_fs(const char full_path[])
 	{
 		if(state.curr_len > 0)
 		{
-			return begins_with_list_item(fs_name, cfg.slow_fs_list);
+			if(starts_with_list_item(fs_name, cfg.slow_fs_list))
+			{
+				return 1;
+			}
 		}
 	}
-	return 0;
+
+	return find_path_prefix_index(full_path, cfg.slow_fs_list) != -1;
 }
 
 int
@@ -365,28 +384,48 @@ traverse_mount_points(mptraverser client, void *arg)
 	return 0;
 }
 
+/* Checks that the str has at least one of comma separated list (the list) items
+ * as a prefix.  Returns non-zero if so, otherwise zero is returned. */
 static int
-begins_with_list_item(const char pattern[], const char list[])
+starts_with_list_item(const char str[], const char list[])
 {
-	const char *p = list - 1;
+	char *const list_copy = strdup(list);
 
-	do
+	char *prefix = list_copy, *state = NULL;
+	while((prefix = split_and_get(prefix, ',', &state)) != NULL)
 	{
-		char buf[128];
-		const char *t;
-		size_t len;
-
-		t = p + 1;
-		p = strchr(t, ',');
-		if(p == NULL)
-			p = t + strlen(t);
-
-		len = snprintf(buf, MIN(p - t + 1, sizeof(buf)), "%s", t);
-		if(len != 0 && strncmp(pattern, buf, len) == 0)
-			return 1;
+		if(starts_with(str, prefix))
+		{
+			break;
+		}
 	}
-	while(*p != '\0');
-	return 0;
+
+	free(list_copy);
+
+	return prefix != NULL;
+}
+
+/* Finds such elemenent of comma separated list of paths (the list) that the
+ * path is prefixed with it.  Returns the index or -1 on failure. */
+static int
+find_path_prefix_index(const char path[], const char list[])
+{
+	char *const list_copy = strdup(list);
+
+	char *prefix = list_copy, *state = NULL;
+	int i = 0;
+	while((prefix = split_and_get(prefix, ',', &state)) != NULL)
+	{
+		if(path_starts_with(path, prefix))
+		{
+			break;
+		}
+		++i;
+	}
+
+	free(list_copy);
+
+	return (prefix == NULL) ? -1 : i;
 }
 
 unsigned int
@@ -441,5 +480,24 @@ S_ISEXE(mode_t mode)
 	return ((S_IXUSR | S_IXGRP | S_IXOTH) & mode);
 }
 
+int
+executable_exists(const char path[])
+{
+	return access(path, X_OK) == 0 && !is_dir(path);
+}
+
+int
+get_exe_dir(char dir_buf[], size_t dir_buf_len)
+{
+	/* This operation isn't supported on *nix-like operation systems. */
+	return 1;
+}
+
+EnvType
+get_env_type(void)
+{
+	return ET_UNIX;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
