@@ -27,19 +27,24 @@
 
 #include <unistd.h> /* getcwd, stat, sysconf */
 
+#include <errno.h> /* errno */
 #include <locale.h> /* setlocale */
-#include <stdio.h> /* fputs() puts() */
-#include <stdlib.h> /* exit() */
+#include <stddef.h> /* NULL */
+#include <stdio.h> /* FILE fclose() fopen() fprintf() fputs() puts()
+                      snprintf() */
+#include <stdlib.h> /* EXIT_SUCCESS exit() system() */
 #include <string.h>
 
 #include "cfg/config.h"
 #include "cfg/info.h"
+#include "engine/cmds.h"
+#include "engine/keys.h"
+#include "engine/mode.h"
+#include "engine/options.h"
 #include "engine/variables.h"
 #include "menus/menus.h"
 #include "modes/modes.h"
-#include "modes/normal.h"
 #include "modes/view.h"
-#include "utils/env.h"
 #include "utils/fs.h"
 #include "utils/fs_limits.h"
 #include "utils/log.h"
@@ -50,14 +55,17 @@
 #include "utils/utils.h"
 #include "background.h"
 #include "bookmarks.h"
+#include "bracket_notation.h"
 #include "builtin_functions.h"
 #include "color_manager.h"
 #include "color_scheme.h"
 #include "commands.h"
 #include "commands_completion.h"
+#include "dir_stack.h"
 #include "filelist.h"
 #include "fileops.h"
 #include "filetype.h"
+#include "fuse.h"
 #include "ipc.h"
 #include "main_loop.h"
 #include "ops.h"
@@ -67,8 +75,8 @@
 #include "registers.h"
 #include "running.h"
 #include "signals.h"
-#include "sort.h"
 #include "status.h"
+#include "term_title.h"
 #include "trash.h"
 #include "ui.h"
 #include "undo.h"
@@ -81,11 +89,17 @@
 #endif
 
 static void quit_on_arg_parsing(void);
+static int undo_perform_func(OPS op, void *data, const char src[],
+		const char dst[]);
 static void parse_recieved_arguments(char *args[]);
 static void remote_cd(FileView *view, const char *path, int handle);
+static int need_to_switch_active_pane(const char lwin_path[],
+		const char rwin_path[]);
 static void load_scheme(void);
 static void convert_configs(void);
 static int run_converter(int vifm_like_mode);
+static void dump_filenames(const FileView *view, FILE *fp, int nfiles,
+		char *files[]);
 
 static void
 show_version_msg(void)
@@ -190,10 +204,11 @@ parse_args(int argc, char *argv[], const char *dir, char *lwin_path,
 #endif
 		else if(!strcmp(argv[x], "-f"))
 		{
-			cfg.vim_filter = 1;
+			curr_stats.file_picker_mode = 1;
 		}
 		else if(!strcmp(argv[x], "--no-configs"))
 		{
+			/* Do nothing. */
 		}
 		else if(!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
 		{
@@ -314,7 +329,9 @@ main(int argc, char *argv[])
 	set_config_paths();
 	reinit_logger();
 
+	/* Commands module also initializes bracket notation and variables. */
 	init_commands();
+
 	init_builtin_functions();
 	update_path_env(1);
 
@@ -330,7 +347,7 @@ main(int argc, char *argv[])
 	 * external programs. */
 	config_filetypes(&external_command_exists);
 	/* This should be called before loading any configuration file. */
-	reset_all_file_associations(curr_stats.env_type == ENVTYPE_EMULATOR_WITH_X);
+	reset_all_file_associations(curr_stats.exec_env_type == EET_EMULATOR_WITH_X);
 
 	init_option_handlers();
 
@@ -346,13 +363,14 @@ main(int argc, char *argv[])
 
 	init_background();
 
+	init_fileops();
+
 	set_view_path(&lwin, lwin_path);
 	set_view_path(&rwin, rwin_path);
 
-	/* Force view switch when path is specified for invisible pane. */
-	if(lwin_path[0] != '\0' && rwin_path[0] == '\0' && curr_view != &lwin)
+	if(need_to_switch_active_pane(lwin_path, rwin_path))
 	{
-		change_window();
+		swap_view_roles();
 	}
 
 	load_initial_directory(&lwin, dir);
@@ -370,7 +388,7 @@ main(int argc, char *argv[])
 
 	colmgr_init(COLOR_PAIRS);
 	init_modes();
-	init_undo_list(&perform_operation, NULL, &ui_cancellation_requested,
+	init_undo_list(&undo_perform_func, NULL, &ui_cancellation_requested,
 			&cfg.undo_levels);
 	load_local_options(curr_view);
 
@@ -430,6 +448,13 @@ main(int argc, char *argv[])
 	return 0;
 }
 
+/* perform_operation() interface adaptor for the undo unit. */
+static int
+undo_perform_func(OPS op, void *data, const char src[], const char dst[])
+{
+	return perform_operation(op, NULL, data, src, dst);
+}
+
 static void
 parse_recieved_arguments(char *args[])
 {
@@ -447,7 +472,7 @@ parse_recieved_arguments(char *args[])
 			&rwin_handle);
 	exec_startup_commands(argc, args);
 
-	if(get_mode() != NORMAL_MODE && get_mode() != VIEW_MODE)
+	if(NONE(vle_mode_is, NORMAL_MODE, VIEW_MODE))
 	{
 		return;
 	}
@@ -468,7 +493,7 @@ parse_recieved_arguments(char *args[])
 		remote_cd(&rwin, rwin_path, rwin_handle);
 	}
 
-	if(lwin_path[0] != '\0' && rwin_path[0] == '\0' && curr_view != &lwin)
+	if(need_to_switch_active_pane(lwin_path, rwin_path))
 	{
 		change_window();
 	}
@@ -483,13 +508,19 @@ remote_cd(FileView *view, const char *path, int handle)
 	char buf[PATH_MAX];
 
 	if(view->explore_mode)
+	{
 		leave_view_mode();
+	}
 
-	if(view == other_view && get_mode() == VIEW_MODE)
+	if(view == other_view && vle_mode_is(VIEW_MODE))
+	{
 		leave_view_mode();
+	}
 
 	if(curr_stats.view)
+	{
 		toggle_quick_view();
+	}
 
 	snprintf(buf, sizeof(buf), "%s", path);
 	exclude_file_name(buf);
@@ -498,6 +529,18 @@ remote_cd(FileView *view, const char *path, int handle)
 	check_path_for_file(view, path, handle);
 }
 
+/* Decides whether active view should be switched based on paths provided for
+ * panes on the command-line.  Returns non-zero if so, otherwise zero is
+ * returned. */
+static int
+need_to_switch_active_pane(const char lwin_path[], const char rwin_path[])
+{
+	/* Forces view switch when path is specified for invisible pane. */
+	return lwin_path[0] != '\0'
+	    && rwin_path[0] == '\0'
+	    && curr_view != &lwin;
+}
+
 /* Loads color scheme.  Converts old format to the new one if needed.
  * Terminates application with error message on error. */
 static void
@@ -505,16 +548,17 @@ load_scheme(void)
 {
 	if(are_old_color_schemes())
 	{
-		if(run_converter(2) != 0)
+		const int err = run_converter(2);
+		if(err != 0)
 		{
 			endwin();
 			fputs("Problems with running vifmrc-converter\n", stderr);
-			exit(0);
+			exit(err);
 		}
 	}
 	if(color_scheme_exists(curr_stats.color_scheme))
 	{
-		load_color_scheme(curr_stats.color_scheme);
+		load_primary_color_scheme(curr_stats.color_scheme);
 	}
 	load_color_scheme_colors();
 }
@@ -526,6 +570,7 @@ static void
 convert_configs(void)
 {
 	int vifm_like_mode;
+	int err;
 
 	if(!query_user_menu("Configuration update", "Your vifmrc will be "
 			"upgraded to a new format.  Your current configuration will be copied "
@@ -546,11 +591,12 @@ convert_configs(void)
 			"configuration file as it is done in vi/vim.  Do you want vifm to "
 			"behave like vi/vim?");
 
-	if(run_converter(vifm_like_mode) != 0)
+	err = run_converter(vifm_like_mode);
+	if(err != 0)
 	{
 		endwin();
 		fputs("Problems with running vifmrc-converter\n", stderr);
-		exit(0);
+		exit(err);
 	}
 
 	show_error_msg("Configuration update", "Your vifmrc has been upgraded to "
@@ -601,5 +647,210 @@ run_converter(int vifm_like_mode)
 #endif
 }
 
+void
+vifm_restart(void)
+{
+	FileView *tmp_view;
+
+	curr_stats.restart_in_progress = 1;
+
+	/* All user mappings in all modes. */
+	clear_user_keys();
+
+	/* User defined commands. */
+	execute_cmd("comclear");
+
+	/* Directory histories. */
+	ui_view_clear_history(&lwin);
+	ui_view_clear_history(&rwin);
+
+	/* All kinds of history. */
+	(void)hist_reset(&cfg.search_hist, cfg.history_len);
+	(void)hist_reset(&cfg.cmd_hist, cfg.history_len);
+	(void)hist_reset(&cfg.prompt_hist, cfg.history_len);
+	(void)hist_reset(&cfg.filter_hist, cfg.history_len);
+	cfg.history_len = 0;
+
+	/* Session status.  Must be reset _before_ options, because options take some
+	 * of values from status. */
+	(void)reset_status(&cfg);
+
+	/* Options of current pane. */
+	reset_options_to_default();
+	/* Options of other pane. */
+	tmp_view = curr_view;
+	curr_view = other_view;
+	load_local_options(other_view);
+	reset_options_to_default();
+	curr_view = tmp_view;
+
+	/* File types and viewers. */
+	reset_all_file_associations(curr_stats.exec_env_type == EET_EMULATOR_WITH_X);
+
+	/* Undo list. */
+	reset_undo_list();
+
+	/* Directory stack. */
+	clean_stack();
+
+	/* Registers. */
+	clear_registers();
+
+	/* Clear all bookmarks. */
+	clear_all_bookmarks();
+
+	/* Reset variables. */
+	clear_variables();
+	init_variables();
+	/* This update is needed as clear_variables() will reset $PATH. */
+	update_path_env(1);
+
+	reset_views();
+	read_info_file(1);
+	save_view_history(&lwin, NULL, NULL, -1);
+	save_view_history(&rwin, NULL, NULL, -1);
+
+	/* Color schemes. */
+	if(stroscmp(curr_stats.color_scheme, DEF_CS_NAME) != 0 &&
+			color_scheme_exists(curr_stats.color_scheme))
+	{
+		if(load_primary_color_scheme(curr_stats.color_scheme) != 0)
+		{
+			load_def_scheme();
+		}
+	}
+	else
+	{
+		load_def_scheme();
+	}
+	load_color_scheme_colors();
+
+	source_config();
+	exec_startup_commands(0, NULL);
+
+	curr_stats.restart_in_progress = 0;
+
+	update_screen(UT_REDRAW);
+}
+
+void
+vifm_try_leave(int write_info, int force)
+{
+	if(!force && bg_has_active_jobs())
+	{
+		if(!query_user_menu("Warning", "Some of backgrounded commands are still "
+					"working.  Quit?"))
+		{
+			return;
+		}
+	}
+
+	unmount_fuse();
+
+	if(write_info)
+	{
+		write_info_file();
+	}
+
+	if(curr_stats.file_picker_mode)
+	{
+		char buf[PATH_MAX];
+		FILE *fp;
+
+		snprintf(buf, sizeof(buf), "%s/vimfiles", cfg.config_dir);
+		fp = fopen(buf, "w");
+		if(fp != NULL)
+		{
+			fclose(fp);
+		}
+		else
+		{
+			LOG_SERROR_MSG(errno, "Can't truncate file: \"%s\"", buf);
+		}
+	}
+
+#ifdef _WIN32
+	system("cls");
+#endif
+
+	set_term_title(NULL);
+	endwin();
+	exit(0);
+}
+
+void _gnuc_noreturn
+vifm_return_file_list(const FileView *view, int nfiles, char *files[])
+{
+	FILE *fp;
+	char filepath[PATH_MAX];
+	int exit_code = EXIT_SUCCESS;
+
+	snprintf(filepath, sizeof(filepath), "%s/vimfiles", cfg.config_dir);
+	fp = fopen(filepath, "w");
+	if(fp != NULL)
+	{
+		dump_filenames(view, fp, nfiles, files);
+		fclose(fp);
+	}
+	else
+	{
+		LOG_SERROR_MSG(errno, "Can't open file for writing: \"%s\"", filepath);
+		exit_code = EXIT_FAILURE;
+	}
+
+	write_info_file();
+
+	endwin();
+	exit(exit_code);
+}
+
+/* Writes list of full paths to files into the file pointed to by fp.  files and
+ * nfiles parameters can be used to supply list of file names in the currecnt
+ * directory of the view.  Otherwise current selection is used if current files
+ * is selected, if current file is not selected it's the only one that is
+ * stored. */
+static void
+dump_filenames(const FileView *view, FILE *fp, int nfiles, char *files[])
+{
+	if(nfiles == 0)
+	{
+		if(!view->dir_entry[view->list_pos].selected)
+		{
+			fprintf(fp, "%s", view->curr_dir);
+			if(view->curr_dir[strlen(view->curr_dir) - 1] != '/')
+			{
+				fprintf(fp, "%s", "/");
+			}
+			fprintf(fp, "%s\n", view->dir_entry[view->list_pos].name);
+		}
+		else
+		{
+			int i;
+			for(i = 0; i < view->list_rows; ++i)
+			{
+				if(view->dir_entry[i].selected)
+				{
+					fprintf(fp, "%s/%s\n", view->curr_dir, view->dir_entry[i].name);
+				}
+			}
+		}
+	}
+	else
+	{
+		int i;
+		for(i = 0; i < nfiles; ++i)
+		{
+			if(is_path_absolute(files[i]))
+			{
+				fprintf(fp, "%s\n", files[i]);
+			}
+			else
+			{
+				fprintf(fp, "%s/%s\n", view->curr_dir, files[i]);
+			}
+		}
+	}
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 : */
