@@ -1,3 +1,4 @@
+#include <stddef.h> /* NULL */
 #include <stdlib.h>
 #include <string.h>
 
@@ -68,18 +69,18 @@ test_skip_goto(void)
 {
 	char *completion;
 
-	reset_completion();
-	assert_int_equal(0, complete_cmd(""));
-	assert_int_equal(7, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(0, complete_cmd("", NULL));
+	assert_int_equal(7, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("comclear", completion);
 	free(completion);
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("command", completion);
 	free(completion);
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("delcommand", completion);
 	free(completion);
 }
@@ -89,15 +90,15 @@ test_skip_abbreviations(void)
 {
 	char *completion;
 
-	reset_completion();
-	assert_int_equal(0, complete_cmd("d"));
-	assert_int_equal(3, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(0, complete_cmd("d", NULL));
+	assert_int_equal(3, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("delcommand", completion);
 	free(completion);
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("delete", completion);
 	free(completion);
 }
@@ -107,21 +108,21 @@ test_dont_remove_range(void)
 {
 	char *completion;
 
-	reset_completion();
-	assert_int_equal(2, complete_cmd("% del"));
-	assert_int_equal(3, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(2, complete_cmd("% del", NULL));
+	assert_int_equal(3, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("delcommand", completion);
 	free(completion);
 
-	reset_completion();
-	assert_int_equal(1, complete_cmd("3del"));
-	assert_int_equal(3, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(1, complete_cmd("3del", NULL));
+	assert_int_equal(3, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("delcommand", completion);
 	free(completion);
 }
@@ -131,15 +132,15 @@ test_dont_remove_cmd(void)
 {
 	char *completion;
 
-	reset_completion();
-	assert_int_equal(7, complete_cmd("% dele "));
-	assert_int_equal(3, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(7, complete_cmd("% dele ", NULL));
+	assert_int_equal(3, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("fastrun", completion);
 	free(completion);
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("followlinks", completion);
 	free(completion);
 }
@@ -149,15 +150,15 @@ test_skip_complete_args(void)
 {
 	char *completion;
 
-	reset_completion();
-	assert_int_equal(17, complete_cmd("% dele fast slow "));
-	assert_int_equal(3, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(17, complete_cmd("% dele fast slow ", NULL));
+	assert_int_equal(3, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("fastrun", completion);
 	free(completion);
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("followlinks", completion);
 	free(completion);
 }
@@ -169,12 +170,12 @@ test_com_completion(void)
 
 	assert_int_equal(0, execute_cmd("command udf a"));
 
-	reset_completion();
-	assert_int_equal(4, complete_cmd("com u"));
-	assert_int_equal(2, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(4, complete_cmd("com u", NULL));
+	assert_int_equal(2, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("udf", completion);
 	free(completion);
 }
@@ -186,12 +187,12 @@ test_delc_completion(void)
 
 	assert_int_equal(0, execute_cmd("command udf a"));
 
-	reset_completion();
-	assert_int_equal(5, complete_cmd("delc u"));
-	assert_int_equal(2, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(5, complete_cmd("delc u", NULL));
+	assert_int_equal(2, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("udf", completion);
 	free(completion);
 }
@@ -201,12 +202,12 @@ test_windo_completion(void)
 {
 	char *completion;
 
-	reset_completion();
-	assert_int_equal(0, complete_cmd("win"));
-	assert_int_equal(2, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(0, complete_cmd("win", NULL));
+	assert_int_equal(2, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("windo", completion);
 	free(completion);
 }
@@ -216,12 +217,12 @@ test_windo_windo_completion(void)
 {
 	char *completion;
 
-	reset_completion();
-	assert_int_equal(12, complete_cmd("windo windo "));
-	assert_int_equal(7, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(12, complete_cmd("windo windo ", NULL));
+	assert_int_equal(7, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("comclear", completion);
 	free(completion);
 }
@@ -231,15 +232,15 @@ test_windo_args_completion(void)
 {
 	char *completion;
 
-	reset_completion();
-	assert_int_equal(6, complete_cmd("windo del"));
-	assert_int_equal(3, get_completion_count());
-	if(get_completion_count() == 0)
+	vle_compl_reset();
+	assert_int_equal(6, complete_cmd("windo del", NULL));
+	assert_int_equal(3, vle_compl_get_count());
+	if(vle_compl_get_count() == 0)
 		return;
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("delcommand", completion);
 	free(completion);
-	completion = next_completion();
+	completion = vle_compl_next();
 	assert_string_equal("delete", completion);
 	free(completion);
 }
@@ -247,9 +248,9 @@ test_windo_args_completion(void)
 static void
 test_no_completion_for_negative_ids(void)
 {
-	reset_completion();
-	assert_int_equal(4, complete_cmd("yank "));
-	assert_int_equal(0, get_completion_count());
+	vle_compl_reset();
+	assert_int_equal(4, complete_cmd("yank ", NULL));
+	assert_int_equal(0, vle_compl_get_count());
 }
 
 static void
@@ -261,33 +262,33 @@ test_udf_completion(void)
 	execute_cmd("command baz b");
 	execute_cmd("command foo c");
 
-	reset_completion();
-	assert_int_equal(5, complete_cmd("comm b"));
-	buf = next_completion();
+	vle_compl_reset();
+	assert_int_equal(5, complete_cmd("comm b", NULL));
+	buf = vle_compl_next();
 	assert_string_equal("bar", buf);
 	free(buf);
-	buf = next_completion();
+	buf = vle_compl_next();
 	assert_string_equal("baz", buf);
 	free(buf);
-	buf = next_completion();
+	buf = vle_compl_next();
 	assert_string_equal("b", buf);
 	free(buf);
 
-	reset_completion();
-	assert_int_equal(5, complete_cmd("comm f"));
-	buf = next_completion();
+	vle_compl_reset();
+	assert_int_equal(5, complete_cmd("comm f", NULL));
+	buf = vle_compl_next();
 	assert_string_equal("foo", buf);
 	free(buf);
-	buf = next_completion();
+	buf = vle_compl_next();
 	assert_string_equal("foo", buf);
 	free(buf);
 
-	reset_completion();
-	assert_int_equal(5, complete_cmd("comm b"));
-	buf = next_completion();
+	vle_compl_reset();
+	assert_int_equal(5, complete_cmd("comm b", NULL));
+	buf = vle_compl_next();
 	assert_string_equal("bar", buf);
 	free(buf);
-	buf = next_completion();
+	buf = vle_compl_next();
 	assert_string_equal("baz", buf);
 	free(buf);
 }
