@@ -1,6 +1,7 @@
 #include "seatest.h"
 
 #include "../../src/engine/keys.h"
+#include "../../src/engine/mode.h"
 #include "../../src/modes/modes.h"
 
 static int
@@ -12,7 +13,6 @@ def_handler(const wchar_t keys)
 static void
 setup(void)
 {
-	static int mode = CMDLINE_MODE;
 	static int mode_flags[]= {
 		MF_USES_REGS | MF_USES_COUNT,
 		MF_USES_INPUT,
@@ -20,7 +20,8 @@ setup(void)
 	};
 
 	clear_keys();
-	init_keys(MODES_COUNT, &mode, mode_flags);
+	init_keys(MODES_COUNT, mode_flags);
+	vle_mode_set(CMDLINE_MODE, VMT_SECONDARY);
 	set_def_handler(CMDLINE_MODE, def_handler);
 	add_user_keys(L"asdf", L"ddd", CMDLINE_MODE, 0);
 }
