@@ -11,24 +11,24 @@ test_space_at_the_end(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fusehome=a\\ b\\ ", &start);
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("a\\ b\\ ", completed);
 	free(completed);
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fusehome=a\\ b ", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("all", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("cpoptions", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fastrun", completed);
 	free(completed);
 }
@@ -39,18 +39,18 @@ test_one_choice_opt(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fuse", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fusehome", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fusehome", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fusehome", completed);
 	free(completed);
 }
@@ -62,19 +62,19 @@ test_one_choice_val(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options(buf, &start);
 	assert_true(start == buf + 5);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("name", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("name", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("name", completed);
 	free(completed);
 }
@@ -85,9 +85,9 @@ test_invalid_input(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fast ?f", &start);
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fast ?f", completed);
 	free(completed);
 }
@@ -98,42 +98,42 @@ test_skip_abbreviations(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("all", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("cpoptions", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fastrun", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fusehome", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("sort", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("sortorder", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("tabstop", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("vifminfo", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("", completed);
 	free(completed);
 }
@@ -144,14 +144,14 @@ test_expand_abbreviations(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fr", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fastrun", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fastrun", completed);
 	free(completed);
 }
@@ -163,15 +163,15 @@ test_after_eq_value_completion(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options(buf, &start);
 	assert_true(start == buf + 9);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("options", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("options", completed);
 	free(completed);
 }
@@ -183,15 +183,15 @@ test_after_meq_value_completion(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options(buf, &start);
 	assert_true(start == buf + 10);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("options", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("options", completed);
 	free(completed);
 }
@@ -203,15 +203,15 @@ test_after_peq_value_completion(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options(buf, &start);
 	assert_true(start == buf + 10);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("options", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("options", completed);
 	free(completed);
 }
@@ -223,19 +223,19 @@ test_set_values_completion(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options(buf, &start);
 	assert_true(start == buf + 13);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("commands", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("cs", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("c", completed);
 	free(completed);
 }
@@ -246,24 +246,24 @@ test_colon(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fusehome:a\\ b\\ ", &start);
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("a\\ b\\ ", completed);
 	free(completed);
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fusehome:a\\ b ", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("all", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("cpoptions", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("fastrun", completed);
 	free(completed);
 }
@@ -274,14 +274,14 @@ test_umbiguous_beginning(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("sort", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("sort", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("sortorder", completed);
 	free(completed);
 }
@@ -292,18 +292,18 @@ test_matching_short_full(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("so", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("sort", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("sortorder", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("so", completed);
 	free(completed);
 }
@@ -317,10 +317,10 @@ test_after_equal_sign_completion_ok(void)
 	optval_t val = { .str_val = "/home/tmp" };
 	set_option("fusehome", val);
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fusehome=", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("/home/tmp", completed);
 	free(completed);
 }
@@ -334,10 +334,10 @@ test_after_equal_sign_completion_spaces_ok(void)
 	optval_t val = { .str_val = "/home directory/tmp" };
 	set_option("fusehome", val);
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fusehome=", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("/home\\ directory/tmp", completed);
 	free(completed);
 }
@@ -351,10 +351,10 @@ test_after_fake_equal_sign_completion_fail(void)
 	optval_t val = { .str_val = "/home/tmp" };
 	set_option("fusehome", val);
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("fusehome=a=", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("a=", completed);
 	free(completed);
 }
@@ -365,10 +365,10 @@ test_all_completion_ok(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("all=", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("", completed);
 	free(completed);
 }
@@ -379,22 +379,22 @@ test_charset_completion_skips_entered_elements(void)
 	const char *start;
 	char *completed;
 
-	reset_completion();
+	vle_compl_reset();
 	complete_options("cpo=a", &start);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("b", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("c", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("", completed);
 	free(completed);
 
-	completed = next_completion();
+	completed = vle_compl_next();
 	assert_string_equal("b", completed);
 	free(completed);
 }
