@@ -1,6 +1,8 @@
 #ifndef VIFM_TESTS__PARSING__ASSERTS_H__
 #define VIFM_TESTS__PARSING__ASSERTS_H__
 
+#include <stdlib.h> /* free() */
+
 /* This should be a macro to see what test has failed. */
 #define ASSERT_OK(str, result) \
 	{ \
@@ -13,6 +15,17 @@
 		var_free(res_var); \
 	}
 
+/* This should be a macro to see what test has failed. */
+#define ASSERT_INT_OK(str, result) \
+	{ \
+		int int_res; \
+		var_t res_var = var_false(); \
+		assert_int_equal(PE_NO_ERROR, parse((str), &res_var)); \
+		int_res = var_to_integer(res_var); \
+		assert_int_equal((result), int_res); \
+		var_free(res_var); \
+	}
+
 /* This should be a macro to see what test has failed. */
 #define ASSERT_FAIL(str, error) \
 	{ \
