@@ -51,6 +51,24 @@ test_concatenation(void)
 	ASSERT_OK("$ENV . $VAR", "NVAR");
 }
 
+static void
+test_non_first_digit_in_name_ok(void)
+{
+	ASSERT_OK("$VAR_NUMBER_1", "AR_NUMBER_1");
+}
+
+static void
+test_first_digit_in_name_fail(void)
+{
+	ASSERT_FAIL("$1_VAR", PE_INVALID_EXPRESSION);
+}
+
+static void
+test_invalid_first_char_in_name_fail(void)
+{
+	ASSERT_FAIL("$#", PE_INVALID_EXPRESSION);
+}
+
 void
 envvar_tests(void)
 {
@@ -63,6 +81,9 @@ envvar_tests(void)
 	run_test(test_trailing_spaces_ok);
 	run_test(test_space_in_the_middle_fail);
 	run_test(test_concatenation);
+	run_test(test_non_first_digit_in_name_ok);
+	run_test(test_first_digit_in_name_fail);
+	run_test(test_invalid_first_char_in_name_fail);
 
 	test_fixture_end();
 }
