@@ -3,6 +3,7 @@
 #include "seatest.h"
 
 #include "../../src/engine/variables.h"
+#include "../../src/utils/env.h"
 #include "../../src/utils/utils.h"
 
 #define VAR_NAME "VAR"
@@ -17,7 +18,7 @@ static void
 test_envvar_remove_on_clear(void)
 {
 	assert_true(getenv(VAR_NAME) == NULL);
-	assert_int_equal(0, let_variable("$" VAR_NAME "='VAL'"));
+	assert_int_equal(0, let_variables("$" VAR_NAME "='VAL'"));
 	assert_true(getenv(VAR_NAME) != NULL);
 
 	clear_variables();
@@ -34,7 +35,7 @@ test_envvar_reset_on_clear(void)
 		assert_string_equal("VAL_A", getenv("VAR_A"));
 	}
 
-	assert_int_equal(0, let_variable("$VAR_A='VAL_2'"));
+	assert_int_equal(0, let_variables("$VAR_A='VAL_2'"));
 	assert_true(getenv("VAR_A") != NULL);
 	if(getenv("VAR_A") != NULL)
 	{
